@@ -1,6 +1,6 @@
 BEGIN {
     $| = 1;
-    print "1..8\n";
+    print "1..13\n";
 }
 END {print "not ok 1\n" unless $loaded;}
 use Algorithm::FastPermute;
@@ -34,7 +34,7 @@ untie @array;
 @array = (qw/a r s e/);
 $i = 0;
 permute {eval {goto foo}; ++$i } @array;
-if ($@ =~ /^Can't "goto" out/) {
+if ($@ =~ /^Can't /) {
     print "ok 7\n";
 } else {
     foo: print "not ok 7\t# $@\n";
@@ -42,6 +42,34 @@ if ($@ =~ /^Can't "goto" out/) {
 
 print ($i == 24 ? "ok 8\n" : "not ok 8\n");
 
+eval {
+  permute {die} @array;
+};
+print ($@ =~ /^Died/ ? "ok 9\n" : "not ok 9\n");
+
+eval { permute {return;} @array; };
+print ($@ eq "" ? "ok 10\n" : "not ok 10\n");
+
+eval { permute {while (1) {return}} @array; };
+print ($@ eq "" ? "ok 11\n" : "not ok 11\n");
+
+eval { permute {@array = ()} @array };
+print ($@ =~ /^Modification of a read-only/ ?
+	"ok 12\n" :
+	"not ok 12\n");
+
+if (!defined &Internals::SvREADONLY) {
+    print "ok 13\t#skip No SvREADONLY\n";
+}
+else {
+    my @array = (1..10);
+    Internals::SvREADONLY(@array, 1);
+    eval { permute {} @array };
+    print ($@ =~ /^(Modification of|Can't permute) a read-only/ ?
+	"ok 13\n" :
+	"not ok 13\t# $@\n");
+}
+
 my $c;
 package TieTest;
 sub TIEARRAY  {bless []}
